/*
FILE:     CifParentChild.C
*/
/*
VERSION:  1.050
*/
/*
DATE:     10/13/2008
*/
/*
  Comments and Questions to: sw-help@rcsb.rutgers.edu
*/
/*
COPYRIGHT 1999-2008 Rutgers - The State University of New Jersey

This software is provided WITHOUT WARRANTY OF MERCHANTABILITY OR
FITNESS FOR A PARTICULAR PURPOSE OR ANY OTHER WARRANTY, EXPRESS OR
IMPLIED.  RUTGERS MAKE NO REPRESENTATION OR WARRANTY THAT THE
SOFTWARE WILL NOT INFRINGE ANY PATENT, COPYRIGHT OR OTHER
PROPRIETARY RIGHT.

The user of this software shall indemnify, hold harmless and defend
Rutgers, its governors, trustees, officers, employees, students,
agents and the authors against any and all claims, suits,
losses, liabilities, damages, costs, fees, and expenses including
reasonable attorneys' fees resulting from or arising out of the
use of this software.  This indemnification shall include, but is
not limited to, any and all claims alleging products liability.
*/
/*
               RCSB PDB SOFTWARE LICENSE AGREEMENT

BY CLICKING THE ACCEPTANCE BUTTON OR INSTALLING OR USING 
THIS "SOFTWARE, THE INDIVIDUAL OR ENTITY LICENSING THE  
SOFTWARE ("LICENSEE") IS CONSENTING TO BE BOUND BY AND IS 
BECOMING A PARTY TO THIS AGREEMENT.  IF LICENSEE DOES NOT 
AGREE TO ALL OF THE TERMS OF THIS AGREEMENT
THE LICENSEE MUST NOT INSTALL OR USE THE SOFTWARE.

1. LICENSE AGREEMENT

This is a license between you ("Licensee") and the Protein Data Bank (PDB) 
at Rutgers, The State University of New Jersey (hereafter referred to 
as "RUTGERS").   The software is owned by RUTGERS and protected by 
copyright laws, and some elements are protected by laws governing 
trademarks, trade dress and trade secrets, and may be protected by 
patent laws. 

2. LICENSE GRANT

RUTGERS grants you, and you hereby accept, non-exclusive, royalty-free 
perpetual license to install, use, modify, prepare derivative works, 
incorporate into other computer software, and distribute in binary 
and source code format, or any derivative work thereof, together with 
any associated media, printed materials, and on-line or electronic 
documentation (if any) provided by RUTGERS (collectively, the "SOFTWARE"), 
subject to the following terms and conditions: (i) any distribution 
of the SOFTWARE shall bind the receiver to the terms and conditions 
of this Agreement; (ii) any distribution of the SOFTWARE in modified 
form shall clearly state that the SOFTWARE has been modified from 
the version originally obtained from RUTGERS.  

2. COPYRIGHT; RETENTION OF RIGHTS.  

The above license grant is conditioned on the following: (i) you must 
reproduce all copyright notices and other proprietary notices on any 
copies of the SOFTWARE and you must not remove such notices; (ii) in 
the event you compile the SOFTWARE, you will include the copyright 
notice with the binary in such a manner as to allow it to be easily 
viewable; (iii) if you incorporate the SOFTWARE into other code, you 
must provide notice that the code contains the SOFTWARE and include 
a copy of the copyright notices and other proprietary notices.  All 
copies of the SOFTWARE shall be subject to the terms of this Agreement.  

3. NO MAINTENANCE OR SUPPORT; TREATMENT OF ENHANCEMENTS 

RUTGERS is under no obligation whatsoever to: (i) provide maintenance 
or support for the SOFTWARE; or (ii) to notify you of bug fixes, patches, 
or upgrades to the features, functionality or performance of the 
SOFTWARE ("Enhancements") (if any), whether developed by RUTGERS 
or third parties.  If, in its sole discretion, RUTGERS makes an 
Enhancement available to you and RUTGERS does not separately enter 
into a written license agreement with you relating to such bug fix, 
patch or upgrade, then it shall be deemed incorporated into the SOFTWARE 
and subject to this Agreement. You are under no obligation whatsoever 
to provide any Enhancements to RUTGERS or the public that you may 
develop over time; however, if you choose to provide your Enhancements 
to RUTGERS, or if you choose to otherwise publish or distribute your 
Enhancements, in source code form without contemporaneously requiring 
end users or RUTGERS to enter into a separate written license agreement 
for such Enhancements, then you hereby grant RUTGERS a non-exclusive,
royalty-free perpetual license to install, use, modify, prepare
derivative works, incorporate into the SOFTWARE or other computer
software, distribute, and sublicense your Enhancements or derivative
works thereof, in binary and source code form.

4. FEES.  There is no license fee for the SOFTWARE.  If Licensee
wishes to receive the SOFTWARE on media, there may be a small charge
for the media and for shipping and handling.  Licensee is
responsible for any and all taxes.

5. TERMINATION.  Without prejudice to any other rights, Licensor
may terminate this Agreement if Licensee breaches any of its terms
and conditions.  Upon termination, Licensee shall destroy all
copies of the SOFTWARE.

6. PROPRIETARY RIGHTS.  Title, ownership rights, and intellectual
property rights in the Product shall remain with RUTGERS.  Licensee 
acknowledges such ownership and intellectual property rights and will 
not take any action to jeopardize, limit or interfere in any manner 
with RUTGERS' ownership of or rights with respect to the SOFTWARE.  
The SOFTWARE is protected by copyright and other intellectual 
property laws and by international treaties.  Title and related 
rights in the content accessed through the SOFTWARE is the property 
of the applicable content owner and is protected by applicable law.  
The license granted under this Agreement gives Licensee no rights to such
content.

7. DISCLAIMER OF WARRANTY.  THE SOFTWARE IS PROVIDED FREE OF 
CHARGE, AND, THEREFORE, ON AN "AS IS" BASIS, WITHOUT WARRANTY OF 
ANY KIND, INCLUDING WITHOUT LIMITATION THE WARRANTIES THAT IT 
IS FREE OF DEFECTS, MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE 
OR NON-INFRINGING.  THE ENTIRE RISK AS TO THE QUALITY AND 
PERFORMANCE OF THE SOFTWARE IS BORNE BY LICENSEE.  SHOULD THE 
SOFTWARE PROVE DEFECTIVE IN ANY RESPECT, THE LICENSEE AND NOT 
LICENSOR ASSUMES THE ENTIRE COST OF ANY SERVICE AND REPAIR.  
THIS DISCLAIMER OF WARRANTY CONSTITUTES AN ESSENTIAL PART OF 
THIS AGREEMENT.  NO USE OF THE PRODUCT IS AUTHORIZED HEREUNDER 
EXCEPT UNDER THIS DISCLAIMER.

8. LIMITATION OF LIABILITY.  TO THE MAXIMUM EXTENT PERMITTED BY
APPLICABLE LAW,  IN NO EVENT WILL LICENSOR BE LIABLE FOR ANY 
INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING, 
WITHOUT LIMITATION, DAMAGES FOR LOSS OF GOODWILL, WORK 
STOPPAGE, COMPUTER FAILURE OR MALFUNCTION, OR ANY AND ALL 
OTHER COMMERCIAL DAMAGES OR LOSSES, EVEN IF ADVISED OF THE
POSSIBILITY THEREOF. 
*/


/*!
** \file CifParentChild.C
**
** \brief Implementation file for CifFile class.
*/


#include <string>
#include <vector>
#include <sstream>

#include "GenString.h"
#include "CifString.h"
#include "ISTable.h"
#include "TableFile.h"
#include "CifParentChild.h"


using std::string;
using std::vector;
using std::ostringstream;


CifParentChild::CifParentChild(ISTable& parChildTable) :
  _parChildTable(parChildTable)
{

}


CifParentChild::~CifParentChild()
{

}


int CifParentChild::CheckParentChild(Block& block, ISTable& catTable,
  ostringstream& log)
{
    int ret = 1;

    vector<string> cifItemNames;
    CifString::MakeCifItems(cifItemNames, catTable.GetName(),
      catTable.GetColumnNames());

    map<string, unsigned int> maxKeyGroups;

    ISTable* keysTableP = CreateKeysTable(cifItemNames, maxKeyGroups);

    if (keysTableP->GetNumRows() == 0)
    {
        delete(keysTableP);

#ifdef JW_DEBUG
        cout << "No parents to check for category " <<  catTable.GetName() << endl;
#endif
        // No parents. Just return.
        return(ret);
    }


    for (map<string, unsigned int>::iterator pos = maxKeyGroups.begin();
      pos != maxKeyGroups.end(); ++pos)
    {
        // pos->first is the parent category name
        // pos->second is the maximum group number for the parent

        if (!block.IsTablePresent(pos->first))
        {
            // Parent table does not exist. Ignore this fact.
#ifdef JW_HACK
            if (pos->first != "chem_comp_atom" &&
              pos->first != "atom_sites_alt")
            {
                log << "++ERROR - In block \"" << block.GetName() <<
                  "\", parent category \"" << pos->first <<
                  "\", of category \"" << catTable.GetName() <<
                  "\", is missing." << endl;
            }
#endif
            continue;
        }

        ISTable* parentCatTableP = block.GetTablePtr(pos->first);
        if (parentCatTableP->GetNumRows() == 0)
        {
#ifdef JW_HACK
            log << "WARN - In block " << block.GetName() << " category " <<
              catTable.GetName() << " parent table empty " << pos->first <<
              endl;
#endif
            // Parent table has no rows. Ignore this fact.
            continue;
        }

        vector<vector<string> > comboKeys;
        vector<string> parKeys;

        GetComboKeys(pos->first, pos->second, *keysTableP, comboKeys,
          parKeys);
#if JW_DEBUG
        cout << "Current category "    <<  catTable.GetName() << endl;
        cout << "Parent category  "    << pos->first << endl;
        cout << "Max group number "    << pos->second << endl;
        //
        for ( unsigned int ii =0; ii < parKeys.size(); ii++)
        {
            cout << "parent  # ii " << ii << " " << parKeys[ii] << endl;
        }

        for ( unsigned int ii =0; ii < comboKeys.size(); ii++)
        {
            for (unsigned int jj =0; jj < comboKeys[ii].size(); jj++)
            {
                cout << " comboKeys[][] " << ii << " " << jj <<  " child = "
                 << comboKeys[ii][jj] << endl;
            }
        }
#endif

        vector<string> parKeyItems;
        for (unsigned int parKeyItemsI = 0; parKeyItemsI < parKeys.size();
          ++parKeyItemsI)
        {
            string tmpItem;
            CifString::GetItemFromCifItem(tmpItem, parKeys[parKeyItemsI]);
            parKeyItems.push_back(tmpItem);
        }

        // Check existence in the parent
        for (unsigned int parKeyI = 0; parKeyI < parKeyItems.size(); ++parKeyI)
        {
            if (!parentCatTableP->IsColumnPresent(parKeyItems[parKeyI]))
            {
                log << "ERROR - In block \"" << block.GetName() <<
                  "\", parent key \"" << parKeyItems[parKeyI] <<
                  "\" is not in parent category \"" << pos->first <<
                  "\"" << endl;
                ret = 0;

                delete(keysTableP);

                return(ret);
            }
        }
        
        for (unsigned int comboKeyI = 0; comboKeyI < comboKeys.size();
          ++comboKeyI)
        {
#ifdef VLAD_PERF
            cout << "Processing parent \"" << pos->first << "\" combo key: " << comboKeyI << " of " <<
              comboKeys.size() << endl;
#endif
#ifdef JW_DEBUG
            cout << "Processing parent \"" << pos->first << "\" combo key: "
                 << comboKeyI + 1 << " of " << comboKeys.size() <<  endl;

            cout << "In category " << catTable.GetName() << " row count  "
                 <<  catTable.GetNumRows() << endl;
#endif
#ifdef JW_HACK
	    int iKeyDif;
	    bool allowMissing = false;
#endif
            for (unsigned int l = 0; l < catTable.GetNumRows(); ++l)
            {
#ifdef JW_DEBUG
                cout << "++ In category " << catTable.GetName() << " row: " << l
                  << " of " << catTable.GetNumRows() << endl;
#endif
#ifdef VLAD_PERF
                cout << "  Processing combo key row: " << l << " of " <<
                  catTable.GetNumRows() << endl;
#endif
                vector<string> parKeyNonEmptyValues;
                vector<string> parKeyAllValues;
                vector<string> parKeyNonEmptyItems;
                vector<string> childKeyNonEmptyItems;
#ifdef JW_HACK
                vector<string> skippedParKeyItems;
		iKeyDif=0;
		allowMissing=false;
#endif
                for (unsigned int keyI = 0; keyI < comboKeys[comboKeyI].size();
                  ++keyI)
                {
                    string itemName;
                    CifString::GetItemFromCifItem(itemName,
                      comboKeys[comboKeyI][keyI]);

                    // loop for all rows
                    const string& parKeyValue = catTable(l, itemName);
                    parKeyAllValues.push_back(parKeyValue);
                    if (CifString::IsEmptyValue(parKeyValue))
                    {
#ifdef JW_HACK
                        // Skip search for null/na/missing values
                        skippedParKeyItems.push_back(parKeyItems[keyI]);
#endif
                    }
                    else
                    {
                        parKeyNonEmptyValues.push_back(parKeyValue);
                        parKeyNonEmptyItems.push_back(parKeyItems[keyI]);
                        childKeyNonEmptyItems.push_back(comboKeys[comboKeyI][keyI]);
                    }
                } // For every key item in a combo key

                if (parKeyNonEmptyValues.empty())
                    continue;

#ifdef JW_HACK
                iKeyDif = comboKeys[comboKeyI].size() - parKeyNonEmptyValues.size();
		//
                // HACK - to deal with real missing values - 
		//
                if (iKeyDif == 1) {
                  if (
		      ((catTable.GetName() == "atom_site") && (pos->first == "entity_poly_seq") && (skippedParKeyItems[0] == "num")) ||
		      ((catTable.GetName() == "atom_site") && (pos->first == "pdbx_poly_seq_scheme") && (skippedParKeyItems[0] == "seq_id")) 

		      ) {
                    allowMissing = true;
		  }

#ifdef JW_DEBUG
		  if (allowMissing) {
                    cout << "++INFO - Allowed exception (keydif=" << iKeyDif << ") " 
			 << " parent " << pos->first 
			 << " skipped parent item(s) " << skippedParKeyItems[0]
			 << " in child " 
			 << catTable.GetName() << " row: " << l+1 << " of " 
			 << catTable.GetNumRows() << endl;
		  }
#endif

		}

                if (iKeyDif == 2) {
                  if ((catTable.GetName() == "pdbx_poly_seq_scheme") && (pos->first == "atom_site") &&
		      ((skippedParKeyItems[0] == "auth_comp_id" &&  skippedParKeyItems[1] == "pdbx_PDB_ins_code") ||
		       (skippedParKeyItems[1] == "auth_comp_id" &&   skippedParKeyItems[0] == "pdbx_PDB_ins_code")))
		    {
		      allowMissing = true;
		    } else if ((catTable.GetName() == "atom_site") && (pos->first == "pdbx_poly_seq_scheme") &&
		      ((skippedParKeyItems[0] == "seq_id" &&   skippedParKeyItems[1] == "pdb_ins_code") ||
		       (skippedParKeyItems[1] == "seq_id" &&   skippedParKeyItems[0] == "pdb_ins_code"))) 
		    {
		      allowMissing = true;
		    }
		  
#ifdef JW_DEBUG
		    if (allowMissing) {
		      cout << "++INFO - Allowed exception (keydif=" << iKeyDif << ") " 
			   << " parent " << pos->first 
			   << " skipped parent item(s) " << skippedParKeyItems[0]
			   << " and " << skippedParKeyItems[1]
			   << " in child " 
			   << catTable.GetName() << " row: " << l+1 << " of " 
			   << catTable.GetNumRows() << endl;
		    }
#endif
		}

                if (iKeyDif == 3) { 
                  if ( (catTable.GetName() == "pdbx_poly_seq_scheme") && (pos->first == "atom_site") &&
		       ((skippedParKeyItems[0] == "auth_seq_id" &&
			 skippedParKeyItems[1] == "auth_comp_id" &&
			 skippedParKeyItems[2] == "pdbx_PDB_ins_code") ||
			(skippedParKeyItems[0] == "auth_seq_id"  &&
			 skippedParKeyItems[1] == "auth_comp_id" &&
			 skippedParKeyItems[2] == "pdbx_PDB_ins_code")))  {
		    allowMissing = true;
		  } else if ((catTable.GetName() == "struct_ref_seq_dif") && (pos->first == "pdbx_poly_seq_scheme") &&
		       ((skippedParKeyItems[0] == "mon_id" &&
			 skippedParKeyItems[1] == "seq_id" &&
			 skippedParKeyItems[2] == "pdb_ins_code") ||
			(skippedParKeyItems[1] == "mon_id"  &&
			 skippedParKeyItems[0] == "seq_id" &&
			 skippedParKeyItems[2] == "pdb_ins_code")))  {
		    allowMissing = true;
		  }
#ifdef JW_DEBUG
		  if (allowMissing) {		   
                    cout << "++INFO - Allowed exception (keydif=" << iKeyDif << ") " 
			 << " parent " << pos->first 
			 << " skipped parent item(s) " << skippedParKeyItems[0]
			 << " and " << skippedParKeyItems[1]
			 << " and " << skippedParKeyItems[2]
			 << " in child " 
			 << catTable.GetName() << " row: " << l+1 << " of " 
			 << catTable.GetNumRows() << endl;
		  }
#endif
                }

                // END HACK -


#ifdef JW_DEBUG
                if (iKeyDif != 0 && !allowMissing)
                {
		  cout << "++INFO - child category (keydif=" << iKeyDif << ") " << catTable.GetName() <<
                      " row: " << l << " of " << catTable.GetNumRows() << endl;
                }
                if (iKeyDif > 0)
                {
                    if (allowMissing)
                    {
                        for ( unsigned int ii =0; ii < skippedParKeyItems.size(); ii++)
                        {
                            cout <<
                              "++INFO - Allowed incomplete parent key in " << pos->first << " "
                              << ii+1 << " of " << skippedParKeyItems.size() <<
                              " -> " << skippedParKeyItems[ii] << endl;
                        }
                    }
                    else
                    {
                        for (unsigned int ii =0;
                          ii < skippedParKeyItems.size(); ii++)
                        {
                            cout << "++INFO - Incomplete parent key in " 
				 << pos->first << " " 
				 << ii+1 << " of " << skippedParKeyItems.size() <<
                              " -> " << skippedParKeyItems[ii] << endl;
                        }
                    }
                }
#endif
#endif

#ifdef JW_HACK
                if ((iKeyDif == 0) || ((iKeyDif > 0) && !allowMissing))
#endif

                {
                    unsigned int searchIndex =
                      parentCatTableP->FindFirst(parKeyNonEmptyValues,
                        parKeyNonEmptyItems);
                    if (searchIndex == parentCatTableP->GetNumRows())
                    {
                        log << "ERROR - In block \"" << block.GetName() 
                          << " unmatched value in " << catTable.GetName() 
                          << " row " << l+1 <<  " in parent category " <<
                          pos->first  << endl;
    
                        for (unsigned int keyValI = 0;
                          keyValI < parKeyNonEmptyValues.size(); ++keyValI)
                        {
                            log << "  \"" << childKeyNonEmptyItems[keyValI] 
                              << "\" -> \"_" << pos->first << "." <<
                              parKeyNonEmptyItems[keyValI] << "\" value =\"" <<
                              parKeyNonEmptyValues[keyValI] << "\"" << endl;
                        }


#ifdef JW_DEBUG
			cout << "ERROR - In block \"" << block.GetName() 
			     << " unsatisfied child value in " << catTable.GetName() 
			     << " row " << l+1 <<  " parent category " << pos->first
			     << endl;
			
			cout << "ERROR - full key \"" << comboKeys[comboKeyI].size()
			     << " realized key size " <<
			  parKeyNonEmptyValues.size() << endl;
			
			for (unsigned int keyValI = 0; keyValI < parKeyNonEmptyValues.size(); ++keyValI)
			  {
			    cout << "  \"" << childKeyNonEmptyItems[keyValI] 
				 << "\"->\"" << pos->first << "." <<
			      parKeyNonEmptyItems[keyValI] << "\" value =\"" <<
			      parKeyNonEmptyValues[keyValI] << "\"" << endl;
			  }

#endif // JW_DEBUG
                    }
                    ret = 0;
                }
            } // For every row in child
        } // For every combo key for one parent
    } // For every parent

    delete(keysTableP);

    return(ret);

}


void CifParentChild::GetParentCifItems(vector<string>& parCifItems,
  const string& cifItemName)
{
    parCifItems.clear();

    vector<string> childCifItem;
    childCifItem.push_back(cifItemName);

    vector<string> childNameCol;
    childNameCol.push_back("child_name");

    vector<unsigned int> parLoc;
    _parChildTable.Search(parLoc, childCifItem, childNameCol);

    for (unsigned int parLocI = 0; parLocI < parLoc.size(); ++parLocI)
    {
        parCifItems.push_back(_parChildTable(parLoc[parLocI], "parent_name"));
    }
}

