/* convert a structure factor file from one format to another */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "sf_convert.h"
#include "util.h"
#include "TableFile.h"
#include "CifFile.h"
#include "CifParserBase.h"

// externals 
char SYMM[20], MTZ2CIF[512], EXECUTE[512];
const char *CCP4,  *URL_USERS_DATA, *USERS_DATA, *CGI_PATH, *PATHTMP;  
int  TEST=0, SINGLE=0, FORMAT=0, DATA_SET=1;
float CELL[6];


int main(int argc, char **argv)
{
    char inpfile[1000], outfile[1000],outfile_tmp[1000], pdbfile[1000];
    char format_inp[20], format_out[20], label_items[512], *temp;
    char cgi_path[512],  pdb_id[256], I_F_type[10];
    int i, j,mtz_cgi=0, cns_cgi=0, label=0, mf=1;
    float flag=0;
    

    PATHTMP = "/tmp/";       //specify the path for computing
    strcpy(pdb_id, "rxxxxsf");
    strcpy(label_items, "");
    strcpy(outfile, "");
    strcpy(pdbfile, "");
    strcpy(format_inp, "");
    
    check_error_message(argc, argv);
    CCP4 = getenv("CCP4");
    
    if((temp = getenv("PROG_VARI")) != NULL) { //only for CGI related
        sprintf(MTZ2CIF,"%s/bin/mtz2cif", temp);
        sprintf(EXECUTE,"%s/bin/sf_convert", temp);
    }else{
        sprintf(MTZ2CIF,"mtz2cif");
        sprintf(EXECUTE,"sf_convert");
//        printf("Error! The PROG_VARI is not set.\n");
    }
    
    for(i=0; i<argc-1; i++){
        if(!strcmp_case(argv[i], "-i")){
            strcpy(format_inp, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-o")){
            strcpy(format_out, argv[i+1]);

        }else if(!strcmp_case(argv[i], "-tmp_path")) {
            PATHTMP = argv[i+1];
            
        }else if(!strcmp_case(argv[i], "-sf")){
            strcpy(inpfile, argv[i+1]);
            file_handle(argv[i+1], inpfile);
            mf=0;
            for(j=i+1; j<argc;j++){ //multifiles to mmcif
                if(argv[j][0]=='-'){
                    i=j-1;
                    break;
                }
                mf++;
            }
            
        }else if(!strcmp_case(argv[i], "-out")){
            strcpy(outfile, argv[i+1]);

            
        }else if(!strcmp_case(argv[i], "-pdb")){
            if(argv[i+1][0] != '-'){
                strcpy(pdbfile, argv[i+1]);
                get_cell_pdb(pdbfile, pdb_id);
            }
            
        }else if(!strcmp_case(argv[i], "-man")||!strcmp_case(argv[i], "-cell")){
            get_cell_man(argv, argc, i, &j);
            i=j;
            
            
        }else if(!strcmp_case(argv[i], "-flag")){
            if(argv[i+1][0] != '-'){
                sscanf(argv[i+1], "%f", &flag);
                if(flag>1)
                    TEST=int(100/flag + 0.5);
                else{
                    TEST=int(100/8 + 0.5);  // 8%
                    printf("The flag is < 1%%, 8%% has been assigned\n");
                }
           }
            
        }else if(!strcmp_case(argv[i], "-sf_type")){ //I or F
            if(argv[i+1][0] != '-')strcpy(I_F_type, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-path")){
            strcpy(cgi_path, argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-pdb_id")){
            sprintf(pdb_id, "r%ssf", argv[i+1]);
            lowerstr(pdb_id);
            
            
        }else if(!strcmp_case(argv[i], "-label")){
            for(j=i+1; j<argc;j++){
                if(argv[j][0]=='-'){
                    i=j-1;
                    break;
                }
                strcat(label_items, argv[j]);
                strcat(label_items, " ");
            }
            label =1;
            
        }else if(!strcmp_case(argv[i], "-mtz_man_html")){ //only for CGI
            mtz_cgi =1;
            if(atoi(argv[i+1])>=1) DATA_SET=atoi(argv[i+1]);
            
            
        }else if(!strcmp_case(argv[i], "-cns_man_html")){ //only for CGI
            cns_cgi =1;
            if(atoi(argv[i+1])>=1) DATA_SET=atoi(argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-valid")){ //only for CGI
            validate_sf_format(argv[i+1]);
            
        }else if(!strcmp_case(argv[i], "-url_users_data")){ //only for CGI
            URL_USERS_DATA=argv[i+1];
            
        }else if(!strcmp_case(argv[i], "-users_data")){ //only for CGI
            USERS_DATA=argv[i+1];
            
        }else if(!strcmp_case(argv[i], "-cgi_bin")){ //only for CGI
            CGI_PATH=argv[i+1];
            
        }else if(!strcmp_case(argv[i], "-remtz")){ //only for CGI
            reformat_mtz_output(argv[i+1],pdb_id);
            exit(0);
            
        }else if(!strcmp_case(argv[i], "-reformat")){ 
            cifparse(argv[i+1],argv[i+1], 80 );
        
        }
    }
    
    if(!strlen(format_inp) && strlen(format_out)){//No input format -i
        guess_sf_format(inpfile,format_inp);
        printf("If the guessed input format (%s) is wrong, please use command:\n", format_inp);
        printf("sf_convert  -i input_format -o output_format -sf data_file\n");
    }
    
    if(!strcmp_case(format_inp, "MMCIF")){
        if(is_cif(inpfile)<4){
            printf("Error! The input file (%s) is not a mmCIF format!\n", inpfile);
            exit(0);
        }
    }
    
    if(!strcmp_case(format_inp, "MTZ")||!strcmp_case(format_out, "MTZ")){
        
        if(!strcmp_case(format_inp, "MTZ") && !is_mtz(inpfile)){
            printf("Error! The input file (%s) is not a MTZ format!\n", inpfile);
            exit(0);
        }
        
//        if((!strcmp_case(CCP4, "$CCP4") && getenv("CCP4")==NULL) ){
            
        if((CCP4=getenv("CCP4"))==NULL ){
            printf("Error! The CCP4 ENV is not set.\n");
            printf("Please source the file 'ccp4.setup' in ccp4-?/include/ and try again.\n");
            exit(0);
        }
    }
    
    
    if(strlen(outfile)<=0) {sprintf(outfile, "%s.%s",inpfile, format_out);}

    if(mf>1){//multifile
        if(!strstr_case(format_inp, "mmcif") && strstr_case(format_out, "mmcif")){
            printf("Multifile conversion. File number=%d\n", mf);
            printf("\nInput File Name = %s : (format=%s)\n",inpfile, format_inp);
            multi_file2mmcif(argc,argv,outfile,format_inp, pdb_id, I_F_type);
            exit(0);
        }
    }
    
    for(i=0; i<argc; i++){
        if(!strcmp_case(argv[i], "-single")){
            SINGLE=1;
        }else if(!strcmp_case(argv[i], "-format")){
            FORMAT=1;
        }
    }
    
    if(FORMAT>0){ 
        guess_sf_format(inpfile, format_inp);
        exit(0);
    }
    
    if(mtz_cgi>0){ //only for CGI
        get_mtz_infor(inpfile, cgi_path, pdb_id);
        exit(0);
    }
    
    if(cns_cgi>0){ //only for CGI
        get_cns_html(inpfile, cgi_path, pdb_id);
        printf("The HTML file is %scns_man_convert.html\n",PATHTMP);
        exit(0);
    }
    
    
    if(label>0){
//convert CNS/MTZ to mmCIF for irregular labels or multiple data sets.
        printf("Manual conversion from %s to mmCIF format ...\n", format_inp);
        if(!strcmp_case(format_inp, "CNS")||!strcmp_case(format_inp, "Xplor"))
            cns_multi_sets(inpfile, outfile, label_items, pdb_id);
        else if(!strcmp_case(format_inp, "MTZ"))
            mtz_multi_sets(inpfile, outfile, label_items, pdb_id); 
        exit(0);
    }
    

//   converting start      
    printf("\nInput File Name = %s : (format=%s)\n",inpfile, format_inp);
    
    if(!strcmp_case(format_inp, "mmcif") && is_cif(inpfile)<3){
        printf("Error! The input file (%s) is not a mmcif format!\n", inpfile);
        exit(0);
    }

    if(!strcmp_case(format_inp, "mmcif") && !strcmp_case(format_out, "mmcif")){
//mmcif to mmcif conversion:
        cif2cif_sf_all(inpfile, outfile,pdb_id);
         
    }else if(!strcmp_case(format_inp, "mmcif") && strcmp_case(format_out, "mmcif")){
//mmcif to non-mmcif conversion:        
        cif_2_various(inpfile, pdbfile, outfile, format_out);
        
    }else if(strcmp_case(format_inp, "mmcif") && !strcmp_case(format_out, "mmcif")){
//non-mmcif to mmcif conversion:        
        various_2_cif(inpfile, outfile,format_inp, pdb_id, I_F_type);
        
    }else {
//non-mmcif to non-mmcif conversion:     (bridged by mmcif)    
        sprintf(outfile_tmp, "%s_tmp.cif", outfile); 
        various_2_cif(inpfile, outfile_tmp, format_inp, pdb_id, I_F_type);
        cif_2_various(outfile_tmp, pdbfile, outfile, format_out);
    }
    
    if(strlen(pdbfile)>0 && !strcmp_case(format_out, "mmcif") ){
        make_complete_SF(pdbfile, outfile);
    }

    validate_sf_format(outfile) ;
    printf("\nOutput File Name = %s : (%s format)\n\n", outfile, format_out);

}

void multi_file2mmcif(int argc,char **argv,char *outfile,char *format_inp,
                      char *pdb_id, char *type)
//combine all together.
{
    char out_tmp[2560],outfile_tmp[256], command[256];
    int i, j;

    for(i=1; i<argc; i++){
        if(!strcmp_case(argv[i], "-sf")){
            for(j=i+1; j<argc; j++){
                if(argv[j][0]=='-'){break;}
                printf("\nProcessing file = %s\n", argv[j]);
                sprintf(outfile_tmp, "%s_%d.tmp", outfile,j);
                
                various_2_cif(argv[j], outfile_tmp, format_inp, pdb_id, type);
                strcat(out_tmp,outfile_tmp);
                strcat(out_tmp," ");
            }
            printf("\nThe output file name = %s\n", outfile);
            
        }
    }
    sprintf(command, "/bin/csh -c \" /bin/cat  %s  > %s\" ", out_tmp,outfile);
    system(command);
    delete_file(outfile_tmp);
    
}

void make_complete_SF(char *pdbfile, char *outfile)
// add categories if exist
{
    char **line1=NULL, **line2=NULL, date[40],pdbid[10], symm[20], tmp[40];
    int i, nline1, nline2, n1=0,n2=0,n3=0,n4=0,n5=0, nn;
    float cell[6], wave;
    FILE *fo;

    
    line1=get_lines_from_file(pdbfile, &nline1);
    line2=get_lines_from_file(outfile, &nline2);
    
    fo=fopen(outfile, "w");

    for(i=0; i<6; i++)cell[i]=0;
    strcpy(date, "?");
    strcpy(symm, "?");
    strcpy(pdbid, "rxxxxsf");
    for(i=0; i<nline1; i++){ //PDB infor.
        if(!strncmp(line1[i], "ATOM  ", 6) || !strncmp(line1[i], "HETATM", 6)) break;
        if (!strncmp(line1[i],"HEADER ", 7) && strlen(line1[i])>62 ){
            sscanf(line1[i]+61, "%s",  tmp);
            if(strlen(tmp)>0){
                lowerstr(tmp);
                sprintf(pdbid,"r%ssf", tmp);
            }   
        }else if(!strncmp(line1[i],"CRYST1 ", 7)){
            sscanf(line1[i]+7, "%f%f %f%f %f%f ",
                   &cell[0], &cell[1], &cell[2], &cell[3], &cell[4], &cell[5]);
            get_string(line1[i],54,64,symm );
            rid_of_front_end_space(symm);
            
        }else if(strstr(line1[i],"REMARK 200  WAVELENGTH OR RANGE        (A) :")){
            sscanf(strchr(line1[i], ':')+1,"%f", &wave);
            
        }
    }
    if(line1 && nline1)free_memory_2d(line1, nline1);
    
    
    for(i=0; i<nline2; i++){ //SF info
        rid_of_front_space(line2[i]);
        if(!strncmp(line2[i], "_audit.", 7)){
            n1=1;
        }else if (!strncmp(line2[i], "_cell.length_a", 14)){
            n2=1;
        }else if (!strncmp(line2[i], "_symmetry.", 9)){
            n3=1;
        }else if (!strncmp(line2[i], "_diffrn_radiation.", 18)){
            n4=1;
        }else if (strstr(line2[i], "_diffrn_radiation_wavelength.wavelength")){
            n5=1;
        }else if(i>100 && !strncmp(line2[i],"data_",5)) break; 
    }

//    printf("n1, ...n4 = %d %d %d %d %d %d :: %s\n", n1, n2,n3,n4,nline1, nline2, outfile );
    
    nn=0;
    for(i=0; i<nline2; i++){ //write SF info
        fprintf(fo, "%s", line2[i]);
        if(!strncmp(line2[i],"data_",5)) {
            nn++;
            if(nn==1 && n1==0) {
                fprintf(fo,"\n#\n");
                fprintf(fo,"_audit.revision_id      1_0\n");
                fprintf(fo,"_audit.creation_date  '%s'\n", date);
                fprintf(fo,"_audit.update_record    'Initial release'\n");  
            }
            if(nn==1 && n2==0) {
                fprintf(fo, "\n#\n_cell.entry_id    %s\n", pdbid);
                fprintf(fo, "_cell.length_a  %.3f\n", cell[0] );
                fprintf(fo, "_cell.length_b  %.3f\n", cell[1]);
                fprintf(fo, "_cell.length_c  %.3f\n", cell[2]);
                fprintf(fo, "_cell.angle_alpha  %.3f\n", cell[3]);
                fprintf(fo, "_cell.angle_beta  %.3f\n", cell[4]);
                fprintf(fo, "_cell.angle_gamma  %.3f\n", cell[5]);
            }
            if(nn==1 && n3==0) {
                fprintf(fo, "\n# \n_symmetry.entry_id    %s\n", pdbid);
                fprintf(fo, "_symmetry.space_group_name_H-M '%s'\n", symm); 
            }
            
            if(nn==1 && n4==0 && n5==0) {
                fprintf(fo, "\n# \n_diffrn_radiation.id   1\n");
                if(wave>0){
                    fprintf(fo, "_diffrn_radiation.pdbx_wavelength    %.5f\n", wave);
                }
                
            }
            
        }
        
    }
    if(line2 && nline2)free_memory_2d(line2, nline2);
    fclose(fo);
}

void usage(int n)
{
    printf("\n");
    printf("Usage: sf_convert  -i input_format -o output_format -sf data_file\n");
    printf("-i     input format: \n");
    printf("       mmCIF, CIF, MTZ, CNS, Xplor, HKL2000, Scalepack, Dtrek, TNT, \n");
    printf("       SHELX, SAINT, EPMR, XSCALE, XPREP, XTALVIEW, X-GEN, XENGEN, \n");
    printf("       MULTAN, MAIN, OTHER.\n");
    printf("-o     output format:\n");
    printf("       mmCIF, MTZ, CNS, TNT, SHELX, EPMR, XTALVIEW, HKL2000, Dtrek,\n");
    printf("       XSCALE, MULTAN, MAIN, OTHER.\n");
    printf("-sf    input structure factor file name.\n");
    printf("\nOptional options:\n");
    printf("-out   followed by output file name (If not given, default by program)\n");
    printf("-pdb   followed by PDB file (must contain symmetry and cell parameters)\n");
    printf("-man   manually type in cells & symmetry (a,b,c,alpha,beta,gamma,p21)\n");
    printf("       (separate each item by a comma ',' without space).\n");
    printf("-flag  followed by a number for Rfree test.\n");
    printf("       (e.g. '-flag 8' means 8%% of reflection selected for free R test)\n");
    printf("-sf_type  followed by I or F. (If not given, guessed by program)\n");
    printf("-label   followed by label name for CNS & MTZ (see 'sf_convert -h')\n");
    printf("-format  guess the format of the SF file (sf_convert -sf sffile -format)\n");
    
    
    printf("\nExample of CNS to mmCIF conversion:  \n");
    printf("    sf_convert  -i cns -o mmcif -sf sf_file_name\n");
    printf("\nExample of Scalepack to MTZ conversion:  \n");
    printf("    sf_convert  -i HKL -o MTZ  -sf sf_file \n");
    printf("    sf_convert  -i HKL -o MTZ  -sf sf_file -man  a,b,c,alpha,beta,gamma,P21\n\n");
    printf("Default data type: F for EPMR, TNT, OTHER, MULTAN, MAIN,XTALVIEW;\n");
    printf("                   I for SHELX,HKL2000,DTREK,XSCALE,XPREP,SAINT;\n\n");
    if(n==0) printf("Type 'sf_convert -h' for special conversion and detail instructions.\n\n");

    if(n==0) exit(0);
    

    printf("==============================================================================\n");
    printf("Note:  \n");
    printf(" 1. If MTZ format is involved, CCP4 must be installed!\n");
    printf(" 2. -pdb or -man must be given for conversion to MTZ file,\n");
    printf("    if input SF file does not contain symmetry and cells.\n");
    printf(" 3. If -man is used, separate each item by a comma ',' without space.\n");
    printf(" 4. If input format is OTHER, you should provide a ASCII file \n");
    printf("    with H, K, L, F, SigmaF separated by space.\n");
    printf(" 5. CIF and mmCIF are for small molecule and macro-molecule format.\n");

    printf("==============================================================================\n\n\
For CNS/Xplor/MTZ to mmCIF, if automatic conversion can not be \n\
performed, or there are multiple datasets in the SF file, the option '-label'\n\
should be provided!\n\
Each label be separated by a comma ',' and each data set be separated by a ':'\n\
\nAll the data labels follow those of CCP4 as bellow. \n\
THE LABELS ARE CASE SENSITIVE!!. \n\
\nIf the labels involved in the '(' or ')', they must be quoted by '?=?'\n\
\nExamples:\n\
 1. convert CNS to mmcif for one data set (if auto conversion failed)\n\
    sf_convert -i cns -o mmcif -sf data_file -label FP=? , SIGFP=?, FREE=?\n\n\
 2. convert CNS to mmcif for two data sets\n\
    sf_convert -i cns -o mmcif -sf data_file -label \n\
               FP=? ,SIGFP=?, I=?, SIGI=? : FP=? , SIGFP=?, I=?, SIGI=? \n\n\
 3. convert MTZ to mmcif for one data set\n\
    sf_convert -i mtz -o mmcif -sf data_file -label FP=? ,SIGFP=?, FREE=?\n\n\
 4. convert MTZ to mmcif for two data set\n\
    sf_convert -i mtz -o mmcif -sf data_file -label FP=? ,SIGFP=?,I=?,SIGI=?\n\
              FREE=? : FP=? , SIGFP=?, I=?, SIGI=? \n\n\
 5. convert MTZ to mmcif (one data set with anomalous)\n\
    sf_convert -i mtz -o mmcif -sf data_file -label FP=? ,SIGFP=? , \n\
              FREE=? , 'F(+)=?' ,'SIGF(+)=?','F(-)=?' ,'SIGF(-)=?'\n\n\
Note: The question marks '?' are corresponding to the labels in the SF file.\n\
\n\
      mmCIF token                    type     data label   \n\
\n\
 _refln.F_meas_au         	      F           FP \n\
 _refln.F_meas_sigma_au               Q           SIGFP \n\
 _refln.intensity_meas    	      J           I     \n\
 _refln.intensity_sigma   	      Q           SIGI   \n\
\n\
 _refln.F_calc            	      F           FC     \n\
 _refln.phase_calc        	      P           PHIC   \n\
 _refln.phase_meas        	      P           PHIB   \n\
 _refln.fom               	      W           FOM    \n\
 \n\
 _refln.pdbx_HL_A_iso                 A       	  HLA   \n\
 _refln.pdbx_HL_B_iso                 A       	  HLB   \n\
 _refln.pdbx_HL_C_iso                 A       	  HLC   \n\
 _refln.pdbx_HL_D_iso                 A       	  HLD   \n\
 \n\
 _refln.pdbx_F_plus         	      G           F(+)   \n\
 _refln.pdbx_F_plus_sigma   	      L           SIGF(+) \n\
 _refln.pdbx_F_minus        	      G           F(-)   \n\
 _refln.pdbx_F_minus_sigma  	      L           SIGF(-)\n\
 _refln.pdbx_anom_difference          D           DP   \n\
 _refln.pdbx_anom_difference_sigma    Q           SIGDP\n\
 _refln.pdbx_I_plus                   K           I(+)   \n\
 _refln.pdbx_I_plus_sigma             M           SIGI(+)\n\
 _refln.pdbx_I_minus                  K           I(-)   \n\
 _refln.pdbx_I_minus_sigma            M           SIGI(-) \n\
\n\
_refln.status            	      I           FREE    \n\
\
");
    
    exit(0);
}

int is_mtz(char *inpfile)
{
    char **line;
    int i, nline=0, mtz=0;

    line=get_lines_from_file(inpfile, &nline);
        
    for(i=0; i<nline; i++){
        if(line[0][0] =='M' && line[0][1] =='T' && line[0][2] =='Z' && line[0][3] ==' ' ) {
            mtz=1;
        }else
            break;
    }
    return mtz;
    
    if(nline)free_memory_2d(line, nline);
}
        
    
void guess_sf_format(char *inpfile, char *format)
// guess various format
{
    char **line,  outfile[256], str[512], tmp[512];
    char s1[80],s2[80],s3[80],s4[80],s5[80],s6[80],s7[80];
    int i, nline, len, n=0, n1, n2,n3,n4,n5,n6,n7, k1,k2,k3, m1,m2, mm1=0;
    float a;
    
    FILE *fout=NULL;

    strcpy(outfile, "sf_format_guess.text");
    fout=fopen(outfile, "w");
    
    line=get_lines_from_file(inpfile, &nline);

    printf("Number of lines =%d\n", nline);
    if(nline<20){
        printf("Error! Input file is wrong!\n");
        fprintf(fout,"Error! Input file is wrong!\n");
        fclose(fout);
        exit(0);
    }
    
    n1=0; //guess mmCIF
    n2=0; //guess CIF
    n3=0; //guess CNS (MAIN)
    n4=0; //guess TNT
    k1=0;k2=0;k3=0;
    m1=0;m2=0;
    
    for(i=0; i<nline; i++){
        if(i>1000) break;
        if(i==0 && line[0][0] =='M' && line[0][1] =='T' && line[0][2] =='Z' && line[0][3] ==' ' ){
            write_format(fout,"MTZ", 10);
            strcpy(format, "MTZ");
            break;
        }
            
        strcpy(tmp,line[i]);
        sscanf(tmp, "%s", str);
        if(!strncmp_case(str, "_refln.", 7)){
             if(!strcmp_case(str, "_refln.F_meas_au")) k1=1;
             if(!strcmp_case(str, "_refln.intensity_meas" ))k2=1;
             if(!strcmp_case(str, "_refln.pdbx_F_plus" ) ||
                 !strcmp_case(str, "_refln.pdbx_I_plus" ))k3=1;
            
            n1++;
        }else if(!strncmp_case(str, "_refln_", 7)){
            if(!strcmp_case(str, "_refln_F_meas" ))k1=1;
            if(!strcmp_case(str, "_refln_F_squared_meas" ))k2=1;
            
            n2++;
        }else if(!strncmp_case(str, "INDE", 4)){
            if(strstr_case(line[i], "FOBS=" ) ||
                 strstr_case(line[i], "FO=" )||
               strstr_case(line[i], "F=" ))m1++;
            
            if(strstr_case(line[i], "IOBS=" ) ||
                 strstr_case(line[i], "IO=" )||
               strstr_case(line[i], "I=" ))m2++;

            if(strstr_case(line[i], "FOBS " ) && strstr_case(line[i], "SIGMA " ) )mm1++;
            
            n3++;
        }else if(!strncmp_case(str, "HKL", 3)){
            sscanf(line[i], "%*s%*s%*s%*s%f", &a);
            if(a<0)m2++;
            n4++;
        }else{
            if(strstr_case(line[i], "FOBS=" ) ||
                 strstr_case(line[i], "FO=" )||
               strstr_case(line[i], "F=" ))m1++;
            
            if(strstr_case(line[i], "IOBS=" ) ||
                 strstr_case(line[i], "IO=" )||
               strstr_case(line[i], "I=" ))m2++;
        }
        
    }
//    printf("m1=m2= %d %d %d %d %d \n", m1, m2, k1, k2,k3);
    
    if(n1>=4){
        if(k1==1 && k2==1){
            write_format(fout,"MMCIF", 2); //I&F

        }else if(k1==1 && k2!=1){
            write_format(fout,"MMCIF", 1); //F
        }else if(k1!=1 && k2==1){
            write_format(fout,"MMCIF", 0); //I
        }else if(k3==1){
            write_format(fout,"MMCIF", 3); //ano
        }else
            write_format(fout,"MMCIF", -1); //nothing

        strcpy(format, "MMCIF");
        
    }else if(n2>=4){
        if(k1==1 && k2==1){
            write_format(fout,"CIF", 2); //I&F
        }else if(k1==1 && k2!=1){
            write_format(fout,"CIF", 1); //F
        }else if(k1!=1 && k2==1){
            write_format(fout,"CIF", 0); //I
        }else
            write_format(fout,"CIF", -1); //nothing
        
        strcpy(format, "CIF");
        
    }else if(n3>=100){
        if(m1>100 && m2>100){
            write_format(fout,"CNS", 2); //I&F
        }else if(m1>100 &&  m2<10){
            write_format(fout,"CNS", 1); //F
        }else if(m2>100 &&  m1<10){
            write_format(fout,"CNS", 0); //I
        }else if(mm1>100 ){
            write_format(fout,"MAIN", 1); //I
        }else
            write_format(fout,"CNS", -1); //nothing

        strcpy(format, "CNS");
        
    }else if(n4>=100){
        if(m2>0)
            write_format(fout,"TNT", 0);
        else
            write_format(fout,"TNT", -2);
        
        strcpy(format, "TNT");
    }
    
    if(strlen(format)>0){
        if(nline)free_memory_2d(line, nline);
        return;
    }
    
// more formats
    n5=0; //guess XSCALE based on head
    n6=0; //guess d*TREK based on head
    n7=0; //guess HKL/SCALEPACK based on head
    for(i=0; i<nline; i++){
        if(i>50) break;
        n=sscanf(line[i], "%s%s%s%s%s%s%s",s1,s2,s3,s4,s5,s6,s7);
            
        if(!strncmp_case(s1, "!SPACE_GROUP_NUMBER=", 20) ||
           !strncmp_case(s1, "!UNIT_CELL_CONSTANTS=", 21) ||
           !strncmp_case(s1, "!ITEM_H=",8) ||
           !strncmp_case(s1, "!ITEM_K=",8) ||
           !strncmp_case(s1, "!ITEM_L=",8) ){
            n5++;
        }else if(!strncmp_case(s1, "CRYSTAL_MOSAICITY=", 18)||
                 !strncmp_case(s1, "CRYSTAL_SPACEGROUP=",19) ||
                 !strncmp_case(s1, "CRYSTAL_UNIT_CELL=",18) ||
                 !strncmp_case(s1, "nH",2) ||
                 !strncmp_case(s1, "nK",2) ||
                 !strncmp_case(s1, "nL",2) 
                 ){
            
            n6++;
        }else if(i==0 && !strcmp_case(s1, "1")){
            n7++;
        }else if((i==1 && !strcmp_case(s1, "-985")) || !strcmp_case(s1, "-987")){
            n7++;
        }else if(i==2 && n==7){
            n7++;
        }  
    }
    if(n5>=5){
        write_format(fout,"XSCALE", 0);
        strcpy(format, "XSCALE");
        
    }else if(n6>=6){
        write_format(fout,"DTREK", 0);
        strcpy(format, "DTREK");
    }else if(n7==3){
        write_format(fout,"HKL", 0);
        strcpy(format, "HKL");
    }

    
    if(strlen(format)>0){
        if(nline)free_memory_2d(line, nline);
        return;
    }

// more formats
    
    k3=0;  //guess saint
    k1=0; k2=0;  //guess SHELX
    n1=0; n2=0; //guess OTHER
    m1=0;m2=0;
    for(i=0; i<nline; i++){
        rid_of_end_space(line[i]);
        n=sscanf(line[i], "%s%s%s%s%s%s",s1,s2,s3,s4,s5,s6);
        if(atoi(s1)<-500 || atoi(s1)>500 ||
           atoi(s2)<-500 || atoi(s2)>500 ||
           atoi(s3)<-500 || atoi(s3)>500 ){
            write_format(fout,"", -1);
        }
        
        len=strlen(line[i]);
        
        if(len==28){
            strncpy(s1, line[i]+12, 8);
            s1[8] = '\0';
            if(atof(s1)<0) m2++; //I
            
            k1++;
        }else if(len==32){
            strncpy(s1, line[i]+12, 8);
            s1[8] = '\0';
            if(atof(s1)<0) m2++; //I
            
            strncpy(s2, line[i]+28, 4);
            s2[4] = '\0';
            if(atoi(s2)==1 || atoi(s2)==-1){ 
                k2++;
            }
        }else if(len>150){
            k3++;
        }
        if(n==5){
            if(atof(s4)<0) m1++; //I
            n1++;
        }else if(n==6){
            if(atof(s4)<0) m1++; //I
            n2++;
        }      
        
    }
    
    if(k3==nline){
        write_format(fout,"SAINT", 0);
        strcpy(format, "SAINT");
        
    }else if(k1==nline || k2==nline){
        if(m2>0)
            write_format(fout,"SHELX", 0);
        else
            write_format(fout,"SHELX", -2);
        strcpy(format, "SHELX");
    }else if(n1==nline || n2==nline){
        if(m2>0)
            write_format(fout,"OTHER", 0);
        else
            write_format(fout,"OTHER", -2);
        
    }else{
        write_format(fout,"", -1);
    }
    if(nline)free_memory_2d(line, nline);
    
}

void  write_format(FILE *fout, const char *format,int key)
{
    int len=0;

    len=strlen(format);
    if(len<1){
        printf("Sorry! format can not be predicted.\n");
        fprintf(fout,"Sorry! format can not be predicted.\n");
    }else{
        printf("Input file FOMRAT=%s\n", format);
        if(key==-1) printf("Data type can not be predicted. Please use the validation programs (SFCHECK, REFMAC ..) to determine the data type!\n");
        if(key==-2) printf("Data type may be Amplitude(F), but you need validation programs (SFCHECK, REFMAC ..) to confirm!\n");
        
        if(key==0) printf("Data type is Intensity (I).\n");
        if(key==1) printf("Data type is Amplitude (F).\n");
        if(key==2) printf("Data type are both Amplitude (F) and Intensity (I).\n");
        if(key==3) printf("Data type are anomolous signal.\n");
        if(key==10) printf("Data type for MTZ file can be auto_converted.\n");
        
        fprintf(fout,"FOMRAT=%s\n", format);
        if(key==-1) fprintf(fout,"Data type can not be predicted. Please use the validation programs (SFCHECK, REFMAC ..) to determine the data type!\n");
        if(key==-2) fprintf(fout,"Data type may be Amplitude(F), but you need validation programs (SFCHECK, REFMAC ..) to confirm!\n");
        if(key==0) fprintf(fout,"Data type is Intensity (I).\n");
        if(key==1) fprintf(fout,"Data type is Amplitude (F).\n");
        if(key==2) fprintf(fout,"Data type are both Amplitude (F) and Intensity (I).\n");
        if(key==3) fprintf(fout,"Data type are anomolous signal.\n");
        if(key==10) fprintf(fout,"Data type for MTZ file is auto_converted.\n");
    }
    
    fclose(fout);
//    exit(0);
}


void get_cell_pdb(char *pdbfile, char *pdb_id)
{
    char str[100], symmetry_in[20], tmp[10];    
    FILE *fpdb;
    
    if((fpdb=fopen(pdbfile, "r"))==NULL){
        open_file_error(pdbfile, "get_cell_pdb");
    }
    
    while(fgets(str, sizeof str, fpdb)!=NULL){
        if (!strncmp(str,"HEADER ", 7)){
            get_string(str,61,66, tmp);
            rid_of_space(tmp);
            if(strlen(tmp)==4){
                lowerstr(tmp);
                strcpy(pdb_id, make_string("r", tmp, "sf"));
            }
            
        }else if (!strncmp(str,"CRYST1 ", 7)){
            sscanf(str+7, "%f%f %f%f %f%f ",
                   &CELL[0], &CELL[1], &CELL[2], &CELL[3], &CELL[4], &CELL[5]);
            
            get_string(str,54,64, symmetry_in);
            space_group_convertor(symmetry_in, SYMM);
//            printf("symmetry_in= %s; standard=%s\n",symmetry_in  , SYMM);
            break;
        }
    }
    fclose(fpdb);

}

void get_cell_man(char **argv, int argc, int in, int *out)
{
    char tmp[256], **line;
    int i, nstr;
    
    if(argv[in+1][0] =='-'){
        printf("Wrong input! It should be '-man  a,b,c,alpha,beta,gamma,SYMM' (no space)\n");
        *out=in+1;
        return;
    }
    
    strcpy(tmp,argv[in+1]);
    
    for(i=in+2; i<argc; i++){
        if(argv[i][0]=='-') break;strcat(tmp," , ");
        strcat(tmp,argv[i]);
    }
    rid_of_space(tmp);
    
    *out=i-1;
    
    line = string_token(tmp,",", &nstr);
    if(nstr!=7){
        printf("Wrong input! Number of input (%d) < 7\n", nstr);
        printf("It should be '-man  a,b,c,alpha,beta,gamma,P21'\n");
        return;
    }
    CELL[0]=atof(line[0]);
    CELL[1]=atof(line[1]);
    CELL[2]=atof(line[2]);
    CELL[3]=atof(line[3]);
    CELL[4]=atof(line[4]);
    CELL[5]=atof(line[5]);
    
    strcpy(SYMM, line[6]);
    if(line) free_memory_2d(line, nstr);
    
//    printf("cell: a= %.2f c= %.2f alpha= %.2f : symmetry_in= %s\n", CELL[0], CELL[2],CELL[3], SYMM);

}

void mtz_multi_sets(char *inpfile, char *outfile, char *label_items, char *pdb_id)
//deal with multiple data sets for MTZ manually    
{
    char str[1000],file_tmp[512],pdbid[20], **dataset;
    int i, n,  nset=0;
    FILE *fout, *ftmp;
    
    if((fout=fopen(outfile,"w"))==NULL){
        open_file_error(outfile,"mtz_multi_sets");
    }
    replace_char(label_items,',',' ');
    
    if(!strstr(label_items, ":")){//single data sets
        get_mtz_man(inpfile, outfile, label_items, pdb_id);
        reformat_mtz_output(outfile,pdb_id);
        printf("\nThe output file = %s\n", outfile);
    }else{
        dataset = string_token(label_items,":", &nset);
        n=0;
        for(i=0; i<nset; i++){
            n=i+1;
            sprintf(file_tmp, "%s_%d",outfile, n);
               
            if(i==0)
                strcpy(pdbid,pdb_id);
            else
                sprintf(pdbid, "%s_%d", pdb_id, i);

            printf("Processing data set = %d\n", n);
               
            get_mtz_man(inpfile, file_tmp, dataset[i] , pdbid);
            reformat_mtz_output(file_tmp, pdbid);

            if((ftmp=fopen(file_tmp,"r"))==NULL){
                open_file_error(file_tmp,"cns_multi_sets");
            }

            while (fgets(str, sizeof str, ftmp) != NULL) {
                fprintf(fout,"%s", str);
            }
            fclose(ftmp);
            delete_file(file_tmp);
        }
        if(dataset) free_memory_2d(dataset, nset);
        printf("\nThe combined output file = %s\n", outfile);
    }
    fclose(fout);
        
}

void get_mtz_man(char *inpfile, char *outfile, char *label_items, char *pdb_id)
{
    char script[512], command[256] ;
    int pid=0, n;
    FILE *fp=NULL;

    pid=getpid();
    sprintf(script, "%smtz2various_%d.scr",PATHTMP, pid);
    
    if((fp=fopen(script, "w"))==NULL){
        printf("Error, file (%s) has no written permission in directory (%s)!\n",
               script, PATHTMP);
        exit(0);
    }
    
    fprintf(fp,"#!/bin/csh \n\n");
    fprintf(fp,"# If you want the log file, replace /dev/null to test.log\n");
    
    fprintf(fp,"%s/bin/mtz2various HKLIN %s HKLOUT %s << EOF> /dev/null\n",
            CCP4, inpfile, outfile);
    fprintf(fp,"OUTPUT CIF data_mtz2cif \n");
    fprintf(fp,"LABIN %s \n", label_items);  
    fprintf(fp,"END\n");
    fprintf(fp,"EOF\n");
    fprintf(fp,"#\n");

    fclose(fp);

    n=check_mtz_label(script);
    if(n==0) {
        printf("Error! No labels are written for MTZ to mmCIF conversion\n");
        return;
    }
    
    
    sprintf(command, "/bin/csh -c \"source %s/include/ccp4.setup; /bin/chmod +x %s ; %s\" >/dev/null", CCP4,script,script);
    system(command);
//    delete_file(script);
    
}


void cns_multi_sets(char *inpfile, char *outfile, char *label_items, char *pdb_id)
//deal with multiple data sets for CNS manually   
{
    char str[1000],file_tmp[512],pdbid[20], **dataset;
    int i, n,  nset=0;
    FILE *fout, *ftmp;
    
    if((fout=fopen(outfile,"w"))==NULL){
        open_file_error(outfile,"cns_multi_sets");
    }
    
    if(strstr(label_items, ":")){//multiple data sets
        dataset = string_token(label_items,":", &nset);
        n=0;
        for(i=0; i<nset; i++){
            n=i+1;
            sprintf(file_tmp, "%s_%d",outfile, n);
            if(i==0)
                strcpy(pdbid,pdb_id);
            else
                sprintf(pdbid, "%s_%d", pdb_id, i);

            printf("Processing data set = %d\n", n);
            get_cns_man(inpfile, file_tmp, dataset[i] , pdbid);

            if((ftmp=fopen(file_tmp,"r"))==NULL){
                open_file_error(file_tmp,"cns_multi_sets");
            }

            while (fgets(str, sizeof str, ftmp) != NULL) {
                fprintf(fout,"%s", str);
            }
            fclose(ftmp);
            delete_file(file_tmp);
        }
        if(dataset) free_memory_2d(dataset, nset);
        printf("\nThe combined output file = %s\n", outfile);
        
    }else{
        get_cns_man(inpfile, outfile, label_items, pdb_id);
        printf("\nThe output file = %s\n", outfile);
    }

    fclose(fout);

}


void get_cns_man(char *inpfile, char *outfile, char *label_items, char *pdb_id)
//manual conversion  (to cif) of CNS format by typing in cns_items
{
    char str[1000], tmp[80], **token , space[10];
    int i, n,  nref=0, ntoken=0, H, K, L;
    
    FILE *finp, *ftmp, *fout;
    
    token = string_token(label_items,",", &ntoken);
    if(ntoken<1) {
        printf("Error! There are not enough data members after -cns_man\n");
        exit(0);
    }
    
    if((fout=fopen(outfile,"w"))==NULL){
        open_file_error(outfile,"get_cns_man");
    }


        
    fprintf(fout,"data_%s\n",pdb_id);
    
    fprintf(fout,"#\n%s\n%s\n%s\n#\n",
            "_audit.revision_id      1_0 ",
            "_audit.creation_date  ?",
            "_audit.update_record    'Initial release' "
            );
    
    fprintf(fout, "#\n");

    fprintf(fout, "loop_\n");
    fprintf(fout, "_refln.crystal_id \n");
    fprintf(fout, "_refln.wavelength_id \n");
    fprintf(fout, "_refln.scale_group_code\n");
    fprintf(fout, "_refln.index_h\n");
    fprintf(fout, "_refln.index_k\n");
    fprintf(fout, "_refln.index_l\n");
    
    strcpy(space," ");
    write_cns_cif_items(fout, space, token, ntoken, 0);//only token names
    
    
    if((finp=fopen(inpfile,"r"))==NULL){
        open_file_error(inpfile,"get_cns_man");
    }

    
    ftmp = tmpfile();
    
    while (fgets(str, sizeof str, finp) != NULL) {
        sscanf(str,"%s",tmp);
            
        if(strstr_case(tmp, "INDE")){
            fprintf(ftmp, "\n");
            nref++;
        }
        n=strlen(str);
        for(i=0; i<n; i++){
            if(isspace(str[i])) str[i]=' '; 
            fprintf(ftmp, "%c", str[i]);
        }
    }
    
    if(nref < 10  ){
        printf("There are too few reflections in the file (%s).\n", inpfile);
        return;
    }
    
    rewind(ftmp);

    while (fgets(str, sizeof str, ftmp) != NULL) {
        rid_of_front_space(str);
        
        if(strstr_case(str, "INDE")){
            sscanf(str,"%*s %d %d %d", &H, &K, &L);
            
            fprintf(fout, "1 1 1 %4d %4d %4d ",H, K, L);
            
            write_cns_cif_items(fout, str, token, ntoken, 1);
            
            fprintf(fout, "\n");
            
        }
    }
    fprintf(fout, "#END OF REFLECTIONS\n");
    fclose(ftmp);
    fclose(finp);
    fclose(fout);
        
    printf("The output file = %s\n",outfile);

    if(token) free_memory_2d(token, ntoken);
    

}

void write_cns_cif_items(FILE *fout, char *str, char **token, int ntoken, int key)
/* write CNS items: 
The label name is the same as CCP4
      mmCIF token                    type     data label                     

 _refln.F_meas_au         	      F           FP 
 _refln.F_meas_sigma_au               Q           SIGFP 
 _refln.intensity_meas    	      J           I      
 _refln.intensity_sigma   	      Q           SIGI   

 _refln.F_calc            	      F           FC     
 _refln.phase_calc        	      P           PHIC   
 _refln.phase_meas        	      P           PHIB   
 _refln.fom               	      W           FOM    
                                                                                           
 _refln.pdbx_HL_A_iso                 A       	  HLA    
 _refln.pdbx_HL_B_iso                 A       	  HLB    
 _refln.pdbx_HL_C_iso                 A       	  HLC    
 _refln.pdbx_HL_D_iso                 A       	  HLD
 
 _refln.pdbx_F_plus         	      G           F(+)   
 _refln.pdbx_F_plus_sigma   	      L           SIGF(+)
 _refln.pdbx_F_minus        	      G           F(-)   
 _refln.pdbx_F_minus_sigma  	      L           SIGF(-)
 _refln.pdbx_anom_difference          D           DANO   
 _refln.pdbx_anom_difference_sigma    Q           SIGDANO
 _refln.pdbx_I_plus                   K           I(+)   
 _refln.pdbx_I_plus_sigma             M           SIGI(+)
 _refln.pdbx_I_minus                  K           I(-)   
 _refln.pdbx_I_minus_sigma            M           SIGI(-)      

_refln.status            	      I           RFREE      
 
*/
{
    char item[80], label[80];
    int i, n;

   
    for(i=0; i<ntoken; i++){
        if(!strstr(token[i], "="))continue;
        
        strcpy(item, strstr(token[i], "=")+1);
        n=strstr(token[i], "=")-token[i];
        strncpy(label,token[i],n); //standard
        label[n]='\0';
        rid_of_space(item);
        rid_of_space(label);
        
        if(key==0){
            printf("data items = %d : label=%s : data item=%s\n",i,label, item);
        }
        strcat(item,"=");

        
        
        if(!strncmp_case(token[i], "FOBS", 4) || !strcmp_case(label, "FP")){
            single_cif_item(fout, str,  key, "_refln.F_meas_au", item, 8);
            
        }else if(!strncmp_case(token[i], "SIGMA", 5) || !strcmp_case(label, "SIGFP")){
            single_cif_item(fout, str,  key, "_refln.F_meas_sigma_au", item, 6);
            
        }else if(!strncmp_case(token[i], "TEST", 4) || !strcmp_case(label, "RFREE")){
            if(key==0){
                fprintf(fout, "_refln.status \n" );
            }else if(key==1){
                n=get_int_after_id(str,item);
                if(n==0){
                    fprintf(fout, " o ");
                }else if(n==1){
                    fprintf(fout, " f ");
                }else
                    fprintf(fout, " o ");
            }
            
       }else if(!strncmp_case(token[i], "IOBS", 4) || !strcmp_case(label, "I")){
            single_cif_item(fout, str,  key, "_refln.intensity_meas", item, 8);
           
        }else if(!strncmp_case(token[i], "SIGI", 4) || !strcmp_case(label, "SIGI")){
            single_cif_item(fout, str,  key, "_refln.intensity_sigma", item, 8);

        }else if(!strncmp_case(token[i], "PHIB", 4) || !strcmp_case(label, "PHIB")){
            single_cif_item(fout, str,  key, "_refln.phase_meas", item, 8);

        }else if(!strncmp_case(token[i], "PHIC", 4)  || !strcmp_case(label, "PHIC")){
            single_cif_item(fout, str,  key, "_refln.phase_calc", item, 8);
            
        }else if(!strncmp_case(token[i], "FOM", 3) || !strcmp_case(label, "FOM") ){
            single_cif_item(fout, str,  key, "_refln.fom", item, 8);

        }else if(!strncmp_case(token[i], "FC", 2) || !strcmp_case(label, "FC")){
            single_cif_item(fout, str,  key, "_refln.F_calc", item, 7);

        }else if(!strncmp_case(token[i], "HLA", 3)  || !strcmp_case(label, "HLA")){
            single_cif_item(fout, str,  key, "_refln.pdbx_HLA ", item, 8);

        }else if(!strncmp_case(token[i], "HLB", 3) || !strcmp_case(label, "HLB") ){
            single_cif_item(fout, str,  key, "_refln.pdbx_HLB", item, 8);

        }else if(!strncmp_case(token[i], "HLC", 3)  || !strcmp_case(label, "HLC")){
            single_cif_item(fout, str,  key, "_refln.pdbx_HLC", item, 8);

        }else if(!strncmp_case(token[i], "HLD", 3) || !strcmp_case(label, "HLD")){
            single_cif_item(fout, str,  key, "_refln.pdbx_HLD", item, 8);
            
        }
        
    }
}

void single_cif_item(FILE *fout, char *str, int key, const char *cif_token, char *item, int fmt)
{
    float value=0;
    
    if(key==0){
        fprintf(fout, "%s \n", cif_token );
    }else if(key==1){
        value=get_float_after_id(str,item);
        if(fmt==6)
            fprintf(fout, "%6.2f ",value );
        else if (fmt==8)
            fprintf(fout, "%8.2f ",value );
        else if (fmt==10)
            fprintf(fout, "%10.2f ",value );
    }
}


void check_error_message(int argc, char **argv)
{
    int key=0, i, j, sf=0;
    const char *option[]=
        {"-i","-o", "-sf", "-out", "-pdb", "-man", "-flag", "-sf_type",
          "-pdb_id","-format", "-mtzdmp", "-label", "END"
        };
    
    
    
    if(argc==1) usage(0);
    if(argc==2 && !strcmp_case(argv[1], "-h")) usage(1);
    
    if(argc<2){
        printf("\nInput error! There are not enough arguments.\n");
        usage(0);
    }
    for(i=1; i<argc; i++){
        if(!strcmp_case(argv[i],"MTZ")) {
            if((CCP4 = getenv("CCP4")) == NULL) {
                printf("Error! you did not set CCP4 environment!\n");
                return;
            }
            break;
        }
    }
    
    for(i=1; i<argc; i++){
            /*
        if( argv[i-1][0]=='-' && argv[i][0]=='-'){
            printf("\nError! Both input arguments (%s ; %s) can not start with '-'!\n\n",
                   argv[i-1],argv[i]);
            printf("Please type 'sf_convert -h' for help!\n\n");
            exit(0);
        }
            */
        
        if(!strcmp_case(argv[i],"-sf")) {
            sf=1;
        }
        
        key=0;
        if( argv[i][0]=='-' ){
            for(j=0; j<100; j++){
                if(!strcmp(option[j], "END")) break;
                if(!strcmp_case(option[j], argv[i])) {
                    key=1;
                    break;
                }
            }
        }
        if(key==0 &&  argv[i][0]=='-' ){
            printf("\nThe input option (%s) is wrong!\n\n", argv[i]);
            printf("Please type 'sf_convert -h' for help!\n\n");
//            exit(0);
        }
    }
    if(sf==0){
        printf("\nError! you did not give the option '-sf'\n\n");
        usage(0);
    }

}

void reformat_mtz_output(char *inpfile, char *pdb_id)
{
    char  **line,str[256];
    FILE  *fout=NULL;
    int i,j,nline;

    
    line=get_lines_from_file(inpfile, &nline);

    
    for(i=0; i<nline; i++){
        rid_of_front_space(line[i]);
        if(line[i][0]=='#'){
           strcpy(line[i],"#\n");
 	}
   }
    
    fout = fopen(inpfile,"w");
    for(i=0; i<nline; i++){    
        if(!strncmp(line[i],"data_", 5)){
            
            fprintf(fout, "data_%s\n", pdb_id);
            fprintf(fout,"#\n%s\n%s\n%s\n#\n",
                    "_audit.revision_id      1_0 ",
                    "_audit.creation_date  ?",
                    "_audit.update_record    'Initial release' "
                    );
	     continue;
        }else if(i>0 && line[i][0]=='#' &&  line[i-1][0]=='#' ){
	     continue;
        }else if(!strncmp(line[i],"_audit.", 7)){
	     for(j=i; j<i+20; j++){

                 
		if(line[j][0]=='#' || !strncmp(line[j],"_cell.", 6)||
		!strncmp(line[j],"loop_", 5)){
                    i=j;
                    break;
                 }
		continue;
              }
        }
        if(strstr_space(line[i], "id     mtz2cif") ){
            replace_first_str(line[i],"mtz2cif", pdb_id, str);
            fprintf(fout, "%s", str);
        }else{
            fprintf(fout, "%s", line[i]);
        }
        
    }
    fprintf(fout,"#END OF REFLECTIONS\n");
    fclose(fout);
    if(nline>0)free_memory_2d(line, nline); 
}

void write_loop(FILE *ftmp, FILE *fout)
{
    int n=0, k=0;
    char str[512], str1[512];
    
    
    while (fgets(str, sizeof str, ftmp) != NULL) {
        if(strchr(str,'#')){
            continue;
        }else {
            n=0;
            fprintf(fout, "%s", str);
            while (fgets(str1, sizeof str1, ftmp) != NULL) {
                n++;
                if(strchr(str1,'#')){
                    k=1;
                    if(n>0) fprintf(fout, "#\n");
                    break;
                }
                
                fprintf(fout, "%s", str1);
            }
        }
        if(n>0 && k==1) break;
    }
}


void delete_tab(char *line)
/* rid of \n */
{
    long len=0, i=0;
    
    len = strlen(line);
    for(i=0; i<len; i++)  
        if(line[i]=='\t')
            line[i]=' ';
    if(iscntrl(line[i-1]))line[i-1]='\0';
}

void cifparse(char *iFile, char *oFile, int width)
// read & write
{
  string diags;
  CifFile * fobjR = NULL;

  
  CifParser* cifParserR = NULL;
  fobjR = new CifFile();
  cifParserR = new CifParser(fobjR, fobjR->GetVerbose());
  cifParserR->Parse(iFile, diags);
  delete(cifParserR);

  if (!diags.empty()) printf(" Diagnostics [ %s] \n", diags.c_str() );
  
  fobjR->Write(oFile, true);
  fobjR->Close();
  if (fobjR) delete fobjR;
  
  if (!diags.empty())
  {
      char* logFileNameP = (char*)malloc(strlen(iFile) + strlen("-parser.log") +
        1);

      memset(logFileNameP, 0, strlen(iFile) + strlen("-parser.log") + 1);
      strcpy(logFileNameP, iFile);
      strcat(logFileNameP, "-parser.log");

      delete_file(logFileNameP);

      free(logFileNameP);
  }
}

int check_mtz_label(char *file)
{
    char **line;
    int nline, i, n=1;

    line=get_lines_from_file(file, &nline);
    
    for(i=0; i<nline; i++){
        if(strstr(line[i], "LABIN ") && !strstr(line[i], "=")){
            n=0;
            break;
        }
    }
    if(nline) free_memory_2d(line, nline);
    return n;
}

    
    
